* Output options: all can be turned on/off by adding or removing the 'NO':
  page number, date, centering, or page breaks, page length and width; 
  OPTIONS nonumber nodate nocenter formdlim='-' pagesize=MAX linesize=MAX spool; 
* Eliminate SAS default titles and names of tables in output (TRACE ON to show);
  TITLE; ODS TRACE OFF; 

* To use FitTest macro;
* FitFewer = Name of ODS InfoCrit table for nested model;
* FitMore  = Name of ODS InfoCrit table for comparison model;
%MACRO FitTest(FitFewer=,FitMore=);
DATA &FitFewer.; LENGTH Name $30.; SET &FitFewer.; Name="&FitFewer."; RUN;
DATA &FitMore.;  LENGTH Name $30.; SET &FitMore.;  Name="&FitMore.";  RUN;
DATA FitCompare; LENGTH Name $30.; SET &FitFewer. &FitMore.; RUN;
DATA FitCompare; SET FitCompare; DevDiff=Lag1(Neg2LogLike)-Neg2LogLike;
     DFdiff=Parms-LAG1(Parms); Pvalue=1-PROBCHI(DevDiff,DFdiff);
     DROP AICC HQIC CAIC; RUN;
TITLE9 "Likelihood Ratio Test for &FitFewer. vs. &FitMore.";
PROC PRINT NOOBS DATA=FitCompare; RUN; TITLE9;
%MEND FitTest;

* To use TotalR2multiv macro;
* DV =        Case-sensitive name of dependent variable;
* PredFewer = Name of OUTPM= data file of predicted outcomes for nested model;
* PredMore =  Name of OUTPM= data file of predicted outcomes for comparison model;
%MACRO TotalR2multiv(DV=,PredFewer=,PredMore=);
PROC SORT DATA=&PredFewer.; BY DV; RUN; PROC SORT DATA=&PredMore.; BY DV; RUN;
PROC CORR NOPRINT NOSIMPLE DATA=&PredFewer. OUTP=CorrFewer; BY DV; VAR pred &DV.; RUN;
PROC CORR NOPRINT NOSIMPLE DATA=&PredMore.  OUTP=CorrMore;  BY DV; VAR pred &DV.; RUN;
DATA CorrFewer; LENGTH Name $30.; SET CorrFewer; Name="&PredFewer."; RUN;
DATA CorrMore;  LENGTH Name $30.; SET CorrMore;  Name="&PredMore.";  RUN;
DATA CorrCompare; LENGTH Name $30.; SET CorrFewer CorrMore; 
     PredCorr=Pred; TotalR2=PredCorr*PredCorr; 
     IF _NAME_="Pred" OR MISSING(_NAME_)=1 THEN DELETE; DROP Pred; RUN;
PROC SORT DATA=CorrCompare; BY DV; RUN;
DATA CorrCompare; SET CorrCompare; TotalR2Diff=TotalR2-LAG1(TotalR2);
	IF Name="&PredFewer." THEN TotalR2Diff=.; 
     KEEP Name DV PredCorr TotalR2 TotalR2Diff; RUN; 
TITLE9 "Total R2 (% Reduction) for &PredFewer. vs. &PredMore.";
PROC PRINT NOOBS DATA=CorrCompare; RUN; TITLE9; 
%MEND TotalR2multiv;

* To use PseudoR2 macro;
* Ncov =     TOTAL # entries in covariance parameter estimates table;
* CovFewer = Name of ODS CovParms table for nested model;
* CovMore =  Name of ODS CovParms table for comparison model;
%MACRO PseudoR2(NCov=,CovFewer=,CovMore=);
DATA &CovFewer.; LENGTH Name $30.; SET &CovFewer.; Name="&CovFewer."; RUN;
DATA &CovMore.;  LENGTH Name $30.; SET &CovMore.;  Name="&CovMore.";  RUN;
DATA CovCompare; LENGTH Name $30.; SET &CovFewer. &CovMore.; RUN;
DATA CovCompare; SET CovCompare; 
     PseudoR2=(LAG&Ncov.(Estimate)-Estimate)/LAG&Ncov.(Estimate); RUN;
DATA CovCompare; SET CovCompare; 
     IF CovParm IN("UN(2,1)","UN(3,1)","UN(3,2)",
			    "UN(4,1)","UN(4,2)","UN(4,3)",
			    "UN(5,1)","UN(5,2)","UN(5,3)","UN(5,4)",
			    "UN(6,1)","UN(6,2)","UN(6,3)","UN(6,4)","UN(6,5)") THEN DELETE; RUN;
TITLE9 "PsuedoR2 (% Reduction) for &CovFewer. vs. &CovMore.";
PROC PRINT NOOBS DATA=CovCompare; RUN; TITLE9;
%MEND PseudoR2;

* Location of original data files - CHANGE THIS;
%LET filesave= C:\Dropbox\18_CLDP945\CLDP945_Example09a;
LIBNAME filesave "&filesave.";

* Example data: DVs = nm3rt, nm5rt, IVs = session, age, educyrs;
* Bring data file into work library;
DATA work.Practice; SET filesave.PracticeEffects; 
* Scaling outcomes in deci-seconds -- Mplus threw up otherwise;
  nm3rt = nm3rt / 100;	LABEL nm3rt= "Number Match 3 RT per 100 ms";
  nm5rt = nm5rt / 100;	LABEL nm5rt= "Number Match 5 RT per 100 ms";
* Creating two slopes for piecewise models;
     IF session = 1 THEN DO; slope12 = 0; slope26 = 0; END;
ELSE IF session = 2 THEN DO; slope12 = 1; slope26 = 0; END;
ELSE IF session > 2 THEN DO; slope12 = 1; slope26 = session-2; END;
LABEL slope12 = "1-2 Early Practice Slope"
      slope26 = "2-6 Later Practice Slope";
* Center time-invariant age;
  age80 = age - 80; LABEL age80 = "Age (0=80)";
* Rename ID for clarity;
  RENAME ID=PersonID; RUN;
PROC SORT DATA=work.Practice; BY PersonID; RUN;

* Trimming data to send just needed variables to Mplus;
DATA work.PracticeMplus; * RETAIN re-orders variables as listed;
	RETAIN PersonID nm3rt nm5rt session slope12 slope26 age80;
	SET work.Practice;
	* Telling it which variables to keep -- handy to use this in Mplus;
	KEEP PersonID nm3rt nm5rt session slope12 slope26 age80;
	* Replace any missing values with -999;
	ARRAY avars(7) PersonID nm3rt nm5rt session slope12 slope26 age80;;
		DO i=1 TO 7; IF avars(i)=. THEN avars(i)=-999; END; DROP i; 
RUN;

* Export to .csv for use in Mplus MLM syntax;
PROC EXPORT DATA=work.PracticeMplus OUTFILE= "&filesave.\practice.csv" 
	DBMS=CSV REPLACE; PUTNAMES=NO; RUN;

* Stack data for multivariate models in SAS;
DATA work.PracticeMultiv; SET work.Practice;
* DV will be used ON CLASS, y is outocme, dv3 and dv5 are "switches";
	DV="nm3rt"; y=nm3rt; dv3=1; dv5=0; OUTPUT;
	DV="nm5rt"; y=nm5rt; dv3=0; dv5=1; OUTPUT;
RUN;

******************************************************************************;
*** NOTE: WE ARE USING ML INSTEAD OF REML TO GET COMPARABLE RESULTS TO MPLUS;
*** I ALSO USED RT/100 BECAUSE OF NUMERIC INSTABILITY PROBLEMS IN MPLUS;
******************************************************************************;

* Send all output to a word file directly;
ODS RTF FILE="&filesave.\CLDP945_Example09a_SAS_Output.rtf" STYLE=HTMLBLUE;

TITLE "Model 1: Multivariate Empty Means, Random Intercept Model";
PROC MIXED DATA=work.PracticeMultiv COVTEST NOCLPRINT NAMELEN=100 IC METHOD=ML;
	CLASS PersonID session DV; * NOINT removes general intercept;
	MODEL y = DV / NOINT SOLUTION DDFM=Satterthwaite;
	RANDOM    DV / G GCORR V VCORR TYPE=UN SUBJECT=PersonID; * Level 2;
	REPEATED  DV / R RCORR TYPE=UN SUBJECT=PersonID*session; * Level 1;
	ODS OUTPUT CovParms=CovEmpty; * Save for pseudo-R2;
RUN;

TITLE1 "Model 2: Multivariate Fixed Piecewise Slopes, Random Intercept Model";
PROC MIXED DATA=work.PracticeMultiv COVTEST NOCLPRINT NAMELEN=100 IC METHOD=ML;
	CLASS PersonID session DV; * Note: no general fixed slope main effects;
	MODEL y = DV DV*slope12 DV*slope26 / NOINT SOLUTION DDFM=Satterthwaite OUTPM=work.PredTime;
	RANDOM    DV / G GCORR TYPE=UN SUBJECT=PersonID;         * Level 2;
	REPEATED  DV / R RCORR TYPE=UN SUBJECT=PersonID*session; * Level 1;
	ODS OUTPUT InfoCrit=FitFixed CovParms=CovFixed; * Save for LRT, pseudo-R2;
RUN;
* Pseudo-R2 for fixed slopes relative to empty model;
%PseudoR2(Ncov=6, CovFewer=CovEmpty, CovMore=CovFixed);

TITLE1 "Model 3: Multivariate Random Piecewise Slopes Model";
PROC MIXED DATA=work.PracticeMultiv COVTEST NOCLPRINT NAMELEN=100 IC METHOD=ML;
	CLASS PersonID session DV; * Note: also no general random slope main effects;
	MODEL y = DV DV*slope12 DV*slope26 / NOINT SOLUTION DDFM=Satterthwaite;
	RANDOM    DV DV*slope12 DV*slope26 / G GCORR TYPE=UN SUBJECT=PersonID; * Level 2;
	REPEATED  DV / R RCORR TYPE=UN SUBJECT=PersonID*session; * Level 1;
	ODS OUTPUT InfoCrit=FitRand CovParms=CovRand; * Save for LRT, pseudo-R2;
	ESTIMATE "DV Diff in Intercept" DV -1 1;
	ESTIMATE "DV Diff in Slope12"   DV*slope12 -1 1;
	ESTIMATE "DV Diff in Slope26"   DV*slope26 -1 1;
	CONTRAST "DV Diff in Both Slopes"  DV*slope12 -1 1, DV*slope26 -1 1 / CHISQ;
RUN;
* LRT against fixed slopes model;
%FitTest(FitFewer=FitFixed, FitMore=FitRand);

TITLE1 "Model 4: Add Age Predicting Multivariate Random Piecewise Slopes";
PROC MIXED DATA=work.PracticeMultiv COVTEST NOCLPRINT NAMELEN=100 IC METHOD=ML;
	CLASS PersonID session DV;
	MODEL y = DV DV*slope12 DV*slope26 DV*age80 DV*slope12*age80 DV*slope26*age80
			/ NOINT SOLUTION DDFM=Satterthwaite OUTPM=PredAge;
	RANDOM    DV DV*slope12 DV*slope26 / G GCORR TYPE=UN SUBJECT=PersonID; * Level 2;
	REPEATED  DV / R RCORR TYPE=UN SUBJECT=PersonID*session; * Level 1;
	ODS OUTPUT InfoCrit=FitAge CovParms=CovAge; * Save for LRT, pseudo-R2;
	ESTIMATE "DV Diff in Age Main Effect" DV*age80 -1 1;
	ESTIMATE "DV Diff in Age*Slope12"     DV*slope12*age80 -1 1;
	ESTIMATE "DV Diff in Age*Slope26"     DV*slope26*age80 -1 1;
     CONTRAST "DV3 DF=3 age effect"       DV*age80  1 0, DV*slope12*age80  1 0, DV*slope26*age80  1 0 / CHISQ;
     CONTRAST "DV5 DF=3 age effect"       DV*age80  0 1, DV*slope12*age80  0 1, DV*slope26*age80  0 1 / CHISQ;
	CONTRAST "DV Diff in 3 Age Effects"  DV*age80 -1 1, DV*slope12*age80 -1 1, DV*slope26*age80 -1 1 / CHISQ;
RUN;
* Pseudo-R2 for age relative to unconditional random slopes model;
%PseudoR2(Ncov=24, CovFewer=CovRand, CovMore=CovAge);
* Total-R2 relative to empty model- note I made a new macro for this;
%TotalR2multiv(DV=y, PredFewer=PredTime, PredMore=PredAge);

* Close output file;
ODS RTF CLOSE;



